/**
 *
 * @author Jens Weller
 * @abstract A C++11 Wordcounting implemenation
 * (c) 2013
 * @license boost software license
 * http://meetingcpp.com
 *
 **/

#include "wordcounter.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <memory>

WordCounter::WordCounter()
{
}

WordCounter::~WordCounter()
{

}

void WordCounter::countWordsThreaded(const std::string file)
{
    std::lock_guard<std::mutex> lock(mymutex);//1
    isrunning = true;
    std::ifstream in(file);//2
    if(!in)
    {
        isrunning = false;
        return;
    }
    in.seekg(0,std::ios::end);//3
    unsigned long size = std::streamoff(in.tellg());
    in.seekg(0,std::ios::beg);

    std::unique_ptr<char[]> data(new char[size]);//4
    in.read(data.get(),size);

    for(char* it= data.get(),* beg = data.get(),*lastwordbegin = nullptr;(it - beg) < size;++it)//5
    {
        if(lastwordbegin && !isWord(*it) )
        {
            ++wordcount[std::string(lastwordbegin,it)];
            lastwordbegin=nullptr;
        }
        else if(!lastwordbegin && isWord(*it))
            lastwordbegin = it;
    }
    isrunning = false;
}

void WordCounter::startCounting(const std::string file)
{
    //std::cout << "startCounting " << file << std::endl;
    if(mythread && mythread->joinable())
    {
        std::cout << "thread still running" << std::endl;
        return;
    }
    mythread.reset( new std::thread(&WordCounter::countWordsThreaded,this, file));
}

void WordCounter::copyWords(wordcontainer &words)
{
    if(!mythread)
        return;
    if(isRunning())
        mythread->join();//wait for finish
    for(std::pair<const std::string,size_t>& pair: wordcount)
    {
        words[pair.first] += pair.second;
        pair.second = 0;
    }
    //wordcount.clear();
}

bool WordCounter::isRunning()
{
    std::lock_guard<std::mutex> lock(mymutex);
    return isrunning;
}

bool WordCounter::isWord(const char &c)
{
    return std::isalnum(c) || c == '_';
}

void WordCounter::join()
{
    if(mythread)
        mythread->join();
}
