/**
 *
 * @author Jens Weller
 * @abstract A C++11 Wordcounting implemenation
 * (c) 2013
 * @license boost software license
 * http://meetingcpp.com
 *
 **/
#ifndef WORDCOUNTER_H
#define WORDCOUNTER_H

#include <string>
#include <thread>
#include <mutex>
#include <unordered_map>
#include <memory>

typedef std::unordered_map<std::string, size_t> wordcontainer;

class WordCounter
{
    std::unique_ptr<std::thread> mythread;

    wordcontainer wordcount;
    bool isrunning=false;
    std::mutex mymutex;
    inline bool isWord(const char& c);
    void countWordsThreaded(const std::string file);
public:
    WordCounter();
    virtual ~WordCounter();
    void startCounting(const std::string file);
    void copyWords(wordcontainer& words);
    bool isRunning();
    void join();
};

#endif // WORDCOUNTER_H
